require 'thor'
require 'slog'


module Anschel

  # Thor's hammer! Like Thor with better logging
  class Mjolnir < Thor

    # Common options for Thor commands
    COMMON_OPTIONS = {
      log: {
        type: :string,
        aliases: %w[ -l ],
        desc: 'Log to file instead of STDOUT',
        default: ENV['ANSCHEL_LOG'] || nil
      },
      debug: {
        type: :boolean,
        aliases: %w[ -v ],
        desc: 'Enable DEBUG-level logging',
        default: ENV['ANSCHEL_DEBUG'] || false
      },
      trace: {
        type: :boolean,
        aliases: %w[ -z ],
        desc: 'Enable TRACE-level logging (overrides DEBUG)',
        default: ENV['ANSCHEL_TRACE'] || false
      }
    }

    # Decorate Thor commands with the options above
    def self.include_common_options
      COMMON_OPTIONS.each do |name, spec|
        option name, spec
      end
    end


    no_commands do

      # Construct a Logger given the command-line options
      def log
        return @logger if defined? @logger
        device = options.log || $stdout
        colorize, prettify = false, false
        colorize, prettify = true, true if device.tty? rescue false
        @logger = Slog.new \
          out: device, colorize: colorize, prettify: prettify
        @logger.level = :debug if options.debug?
        @logger.level = :trace if options.trace?
        @logger
      end

      # Write out a temporary properties file for Log4j
      def setup_log4j config
        config ||= {}
        path     = config.delete(:path)    || '/dev/stdout'
        pattern  = config.delete(:pattern) || '%p [%d] (%c) %m%n'
        Tempfile.open('anschel_log4j') do |f|
          log4j = %Q|
            log4j.rootLogger=INFO, A1
            log4j.appender.A1=org.apache.log4j.RollingFileAppender
            log4j.appender.A1.File=%s
            log4j.appender.A1.layout=org.apache.log4j.PatternLayout
            log4j.appender.A1.layout.ConversionPattern=%s
          | % [ path, pattern ]
          f.write log4j.gsub(/^\s+/,'')
          f.rewind
          org.apache.log4j.PropertyConfigurator.configure(f.path)
        end
      end

    end
  end
end